<?php

namespace Cewi\Excel\View;

use Cake\Core\Configure;
use Cake\Core\Exception\Exception;
use Cake\Event\EventManager;
use Cake\Network\Request;
use Cake\Network\Response;
use Cake\Utility\Text;
use Cake\View\View;
use PhpOffice\PhpSpreadsheet\Cell\AdvancedValueBinder;
use PhpOffice\PhpSpreadsheet\Cell\Cell;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Spreadsheet;

/**
 * @package  Cake.View
 */
class ExcelView extends View
{

    /**
     * \PhpOffice\PhpSpreadsheet\Spreadsheet instance
     *
     * @var \PhpOffice\PhpSpreadsheet\Spreadsheet
     */
    public $PHPSpreadsheet = null;

    /**
     * Filename
     *
     * @var string
     */
    private $__filename;

    /**
     * The subdirectory.  Excel views are always in xlsx.
     *
     * @var string
     */
    public $subDir = 'xlsx';

    /**
     * Pointer to the active sheet in the workbook
     *
     * @var integer
     */
    public $currentSheetIndex = null;

    /**
     * Constructor
     *
     * @param \Cake\Network\Request $request Request instance.
     * @param \Cake\Network\Response $response Response instance.
     * @param \Cake\Event\EventManager $eventManager Event manager instance.
     * @param array $viewOptions View options. See View::$_passedVars for list of
     *   options which get set as class properties.
     *
     * @throws \Cake\Core\Exception\Exception
     */
    public function __construct(Request $request = null, Response $response = null, EventManager $eventManager = null, array $viewOptions = [])
    {
        parent::__construct($request, $response, $eventManager, $viewOptions);

        if (isset($viewOptions['templatePath']) && $viewOptions['templatePath'] == 'Error') {
            $this->layoutPath = null;
            $this->subDir = null;
            $response->withType('html');

            return;
        }

        // intitialize \PhpOffice\PhpSpreadsheet\Spreadsheet-Object
        Cell::setValueBinder(new AdvancedValueBinder());
        $this->PHPSpreadsheet = new Spreadsheet();

        $this->currentSheetIndex = 0;
    }

    /**
     * Initialization hook method.
     * Load the Helper
     */
    public function initialize()
    {
        parent::initialize();
        $this->setLayout('default');
        $this->loadHelper('Cewi/Excel.Excel');
    }

    /**
     * [render description]
     *
     * @param  [type] $action [description]
     * @param  [type] $layout [description]
     * @param  [type] $file   [description]
     * @return [type]         [description]
     */
    public function render($action = null, $layout = null, $file = null)
    {
        $content = parent::render($action, false, $file);
        if ($this->response->getType() == 'text/html') {
            return $content;
        }

        $content = $this->__output();
        $this->Blocks->set('content', $content);

        $this->response = $this->response->withDownload($this->getFilename());

        return $this->Blocks->get('content');
    }

    /**
     * Generates the binary excel data
     *
     * @todo find a way to set date format for generated cells
     * @return string
     * @throws CakeException If the excel writer does not exist
     */
    private function __output()
    {

        // Remove initially created empty sheet
        $this->currentSheetIndex = $this->PHPSpreadsheet->getIndex($this->PHPSpreadsheet->getSheetByName('Worksheet'));
        $this->PHPSpreadsheet->removeSheetByIndex($this->currentSheetIndex);

        $this->PHPSpreadsheet->getProperties()->setCreator(Configure::read('App.author'));
        $this->PHPSpreadsheet->getProperties()->setDescription('generated by ' . Configure::read('App.title'));
        $this->PHPSpreadsheet->getProperties()->setKeywords("office 2007 openxml php");

        ob_start();

        // For now we're creating Excel2007+ - Files @TODO: Make this configurable
        $writer = IOFactory::createWriter($this->PHPSpreadsheet, 'Xlsx');

        if (!isset($writer)) {
            throw new Exception('Excel writer not found');
        }

        $writer->setPreCalculateFormulas(false);
        $writer->setIncludeCharts(true);
        $writer->save('php://output');

        $output = ob_get_clean();

        return $output;
    }

    /**
     * Set the Name of Excel-File
     *
     * @param string $filename
     */
    public function setFilename($filename)
    {
        $this->__filename = $filename;
    }

    /**
     * Get the Name of Excel-File
     *
     * @return string
     */
    public function getFilename()
    {
        if (!empty($this->__filename)) {
            return $this->__filename . '.xlsx';
        }
        return Text::slug(str_replace('.xlsx', '', $this->request->getRequestTarget())) . '.xlsx';
    }

}
